@@ -22,7 +22,7 @@ been successfully completed.
 ::
 
     f107 = pysat.Instrument('sw', 'f107', tag='historic')
-    f107.download(start=f107.lasp_stime, stop=f107.today(), freq='MS')
+    f107.download(start=f107.lasp_stime, stop=f107.today())
     f107.load(date=f107.lasp_stime, end_date=f107.today())
 
 
@@ -57,7 +57,6 @@ available from the pyast.Instrument object is not appropriate for '45day' data.
 
 import datetime as dt
 import ftplib
-import json
 import numpy as np
 import os
 import pandas as pds
@@ -68,6 +67,7 @@ import warnings
 
 from pysatSpaceWeather.instruments.methods import f107 as mm_f107
 from pysatSpaceWeather.instruments.methods import general
+from pysatSpaceWeather.instruments.methods import lisird
 
 logger = pysat.logger
 
@@ -104,7 +104,7 @@ _test_dates = {'': {'historic': dt.datetime(2009, 1, 1),
                     '45day': tomorrow}}
 
 # Other tags assumed to be True
-_test_download_travis = {'': {'prelim': False}}
+_test_download_ci = {'': {'prelim': False}}
 
 # ----------------------------------------------------------------------------
 # Instrument methods
@@ -124,6 +124,26 @@ def init(self):
     if self.tag == 'historic':
         self.lasp_stime = lasp_stime
 
+    # Raise Deprecation warnings
+    if self.tag in ['daily', 'prelim']:
+        # This tag loads more than just F10.7 data, and the behaviour will be
+        # deprecated in v0.1.0
+        warnings.warn("".join(["Upcoming structural changes will prevent ",
+                               "Instruments from loading multiple data sets in",
+                               " one Instrument. In version 0.1.0+ the SSN, ",
+                               "solar flare, and solar mean field data will be",
+                               " accessable from the `sw_ssn`, `sw_flare`, ",
+                               "and `sw_sbfield` Instruments."]),
+                      DeprecationWarning, stacklevel=2)
+    elif self.tag == '45day':
+        # This tag loads more than just F10.7 data, and the behaviour will be
+        # deprecated in v0.1.0
+        warnings.warn("".join(["Upcoming structural changes will prevent ",
+                               "Instruments from loading multiple data sets in",
+                               " one Instrument. In version 0.1.0+ the Ap will",
+                               " be accessable from the `sw_ap` Instrument."]),
+                      DeprecationWarning, stacklevel=2)
+
     return
 
 
@@ -206,7 +226,6 @@ def load(fnames, tag='', inst_id=''):
                       meta.labels.min_val: 0,
                       meta.labels.max_val: 400}
     elif tag == 'historic':
-
         # LASP updated file format in June, 2022. Minimize impact downstream by
         # continuing use of `f107` as primary data product.
         if 'f107_adjusted' in data.columns:
@@ -237,6 +256,10 @@ def load(fnames, tag='', inst_id=''):
                 meta.labels.desc: meta['f107_adjusted', meta.labels.desc]}
 
     elif tag == 'daily' or tag == 'prelim':
+        # Update the allowed types for the fill value
+        meta.labels.label_type['fill_val'] = (float, int, np.float64,
+                                              np.int64, str)
+
         meta['ssn'] = {meta.labels.units: '',
                        meta.labels.name: 'Sunspot Number',
                        meta.labels.notes: '',
@@ -375,9 +398,9 @@ def list_files(tag='', inst_id='', data_path='', format_str=None):
             orig_files = out_files.sort_index().copy()
             new_files = list()
 
-            for orig in orig_files.iteritems():
+            for orig in orig_files.items():
                 # Version determines each file's valid length
-                version = int(orig[1].split("_v")[1][0])
+                version = np.int64(orig[1].split("_v")[1][0])
                 doff = pds.DateOffset(years=1) if version == 2 \
                     else pds.DateOffset(months=3)
                 istart = orig[0]
@@ -455,61 +478,16 @@ def download(date_array, tag, inst_id, data_path, update_files=False):
     if tag == 'historic':
         # Test the date array, updating it if necessary
         if date_array.freq != 'MS':
-            warnings.warn(''.join(['Historic F10.7 downloads should be invoked',
-                                   " with the `freq='MS'` option."]))
             date_array = pysat.utils.time.create_date_range(
                 dt.datetime(date_array[0].year, date_array[0].month, 1),
                 date_array[-1], freq='MS')
 
         # Download from LASP, by month
-        for dl_date in date_array:
-            # Create the name to which the local file will be saved
-            str_date = dl_date.strftime('%Y-%m')
-            data_file = os.path.join(data_path,
-                                     'f107_monthly_{:s}.txt'.format(str_date))
+        freq = pds.DateOffset(months=1, seconds=-1)
+        lisird.download(date_array, data_path, 'f107_monthly_', '%Y-%m',
+                        'noaa_radio_flux', freq, update_files,
+                        {'f107_adjusted': -99999.0, 'f107_observed': -99999.0})
 
-            if update_files or not os.path.isfile(data_file):
-                # Set the download webpage
-                dstr = ''.join(['http://lasp.colorado.edu/lisird/latis/dap/',
-                                'noaa_radio_flux.json?time%3E=',
-                                dl_date.strftime('%Y-%m-%d'),
-                                'T00:00:00.000Z&time%3C=',
-                                (dl_date + pds.DateOffset(months=1)
-                                 - pds.DateOffset(days=1)).strftime('%Y-%m-%d'),
-                                'T00:00:00.000Z'])
-
-                # The data is returned as a JSON file
-                req = requests.get(dstr)
-
-                # Process the JSON file
-                if req.text.find('Gateway Timeout') >= 0:
-                    raise IOError(''.join(['Gateway timeout when requesting ',
-                                           'file using command: ', dstr]))
-
-                raw_dict = json.loads(req.text)['noaa_radio_flux']
-                data = pds.DataFrame.from_dict(raw_dict['samples'])
-                if data.empty:
-                    warnings.warn("no data for {:}".format(dl_date),
-                                  UserWarning)
-                else:
-                    # The file format changed over time
-                    try:
-                        # This is the new data format
-                        times = [dt.datetime.strptime(time, '%Y%m%d')
-                                 for time in data.pop('time')]
-                    except ValueError:
-                        # Accepts old file formats
-                        times = [dt.datetime.strptime(time, '%Y %m %d')
-                                 for time in data.pop('time')]
-                    data.index = times
-
-                    # Replace fill value with NaNs
-                    for var in data.columns:
-                        idx, = np.where(data[var] == -99999.0)
-                        data.iloc[idx, :] = np.nan
-
-                    # Create a local CSV file
-                    data.to_csv(data_file, header=True)
     elif tag == 'prelim':
         ftp = ftplib.FTP('ftp.swpc.noaa.gov')  # Connect to host, default port
         ftp.login()  # User anonymous, passwd anonymous
@@ -652,9 +630,9 @@ def download(date_array, tag, inst_id, data_path, update_files=False):
         # String data is the forecast value for the next three days
         raw_data = req.text.split('10cm_flux:')[-1]
         raw_data = raw_data.split('\n')[1]
-        val1 = int(raw_data[24:27])
-        val2 = int(raw_data[38:41])
-        val3 = int(raw_data[52:])
+        val1 = np.int64(raw_data[24:27])
+        val2 = np.int64(raw_data[38:41])
+        val3 = np.int64(raw_data[52:])
 
         # Put data into nicer DataFrame
         data = pds.DataFrame([val1, val2, val3], index=times, columns=['f107'])
