@@ -8,7 +8,9 @@ platform
 name
     'kp'
 tag
-    - '' Standard Kp data
+    - '' Deprecated, mixed definitive and nowcast Kp data from GFZ
+    - 'def' Definitive Kp data from GFZ
+    - 'now' Nowcast Kp data from GFZ
     - 'forecast' Grab forecast data from SWPC (next 3 days)
     - 'recent' Grab last 30 days of Kp data from SWPC
 inst_id
@@ -59,29 +61,28 @@ of the National Science Foundation.
 """
 
 import datetime as dt
-import ftplib
 import numpy as np
 import os
 import pandas as pds
 import requests
-import sys
+import warnings
 
 import pysat
 
 from pysatSpaceWeather.instruments.methods import general
 from pysatSpaceWeather.instruments.methods import kp_ap
 
-logger = pysat.logger
-
 # ----------------------------------------------------------------------------
 # Instrument attributes
 
 platform = 'sw'
 name = 'kp'
-tags = {'': '',
+tags = {'': 'Deprecated, mixed definitive and nowcast Kp data from GFZ',
+        'def': 'Definitive Kp data from GFZ',
+        'now': 'Nowcast Kp data from GFZ',
         'forecast': 'SWPC Forecast data next (3 days)',
         'recent': 'SWPC provided Kp for past 30 days'}
-inst_ids = {'': ['', 'forecast', 'recent']}
+inst_ids = {'': list(tags.keys())}
 
 # Generate todays date to support loading forecast data
 now = dt.datetime.utcnow()
@@ -91,12 +92,13 @@ today = dt.datetime(now.year, now.month, now.day)
 # Instrument test attributes
 
 # Set test dates
-_test_dates = {'': {'': dt.datetime(2009, 1, 1),
+_test_dates = {'': {'def': dt.datetime(2009, 1, 1),
+                    'now': dt.datetime(2020, 1, 1),
                     'forecast': today + dt.timedelta(days=1),
                     'recent': today}}
 
 # Other tags assumed to be True
-_test_download_travis = {'': {'': False}}
+_test_download_ci = {'': {'': False}}
 
 # ----------------------------------------------------------------------------
 # Instrument methods
@@ -109,7 +111,18 @@ def init(self):
 
     self.acknowledgements = kp_ap.acknowledgements(self.name, self.tag)
     self.references = kp_ap.references(self.name, self.tag)
-    logger.info(self.acknowledgements)
+    pysat.logger.info(self.acknowledgements)
+
+    if self.tag in ["def", "now"]:
+        # This tag loads more than just Kp data, and the behaviour will be
+        # deprecated in v0.1.0
+        warnings.warn("".join(["Upcoming structural changes will prevent ",
+                               "Instruments from loading multiple data sets ",
+                               "in one Instrument. In version 0.1.0+ the Ap ",
+                               "and Cp data will be accessable from the ",
+                               "`sw_ap` and `sw_cp` Instruments."]),
+                      DeprecationWarning, stacklevel=2)
+
     return
 
 
@@ -146,10 +159,22 @@ def load(fnames, tag='', inst_id=''):
     ----
     Called by pysat. Not intended for direct use by user.
 
+    Warnings
+    --------
+    tag '' has been deprecated, will be removed in version 0.2.0+
+
     """
 
     meta = pysat.Meta()
     if tag == '':
+        # This data type has been deprecated due to changes at GFZ
+        warnings.warn("".join(["Changes at the GFZ database have led to this",
+                               " data type being deprecated. Switch to using",
+                               " 'def' for definitive Kp or 'now' for Kp ",
+                               "nowcasts from GFZ. Load support will be ",
+                               "removed in version 0.2.0+"]),
+                      DeprecationWarning, stacklevel=2)
+
         # Kp data stored monthly, need to return data daily.  The daily date is
         # attached to filename.  Parse off the last date, load month of data,
         # and downselect to the desired day
@@ -184,7 +209,7 @@ def load(fnames, tag='', inst_id=''):
                 temp = fin.readlines()
 
             if len(temp) == 0:
-                logger.warn('Empty file: {:}'.format(fname))
+                pysat.logger.warn('Empty file: {:}'.format(fname))
                 continue
 
             # This file has a different format if it is historic or a file that
@@ -229,7 +254,8 @@ def load(fnames, tag='', inst_id=''):
 
             # Kp comes in non-user friendly values like 2-, 2o, and 2+. Relate
             # these to 1.667, 2.0, 2.333 for processing and user friendliness
-            first = np.array([float(str_val[0]) for str_val in data_series])
+            first = np.array([np.float64(str_val[0])
+                              for str_val in data_series])
             flag = np.array([str_val[1] for str_val in data_series])
 
             ind, = np.where(flag == '+')
@@ -243,17 +269,47 @@ def load(fnames, tag='', inst_id=''):
             result = pds.DataFrame()
 
         fill_val = np.nan
-    elif tag == 'forecast':
-        # Load forecast data
-        all_data = []
-        for fname in fnames:
-            result = pds.read_csv(fname, index_col=0, parse_dates=True)
-            all_data.append(result)
+    elif tag in ['def', 'now']:
+        # Load the definitive or nowcast data. The Kp data stored in yearly
+        # files, and we need to return data daily.  The daily date is
+        # attached to filename.  Parse off the last date, load month of data,
+        # and downselect to the desired day
+        data = pds.DataFrame()
 
-        result = pds.concat(all_data)
-        fill_val = -1
-    elif tag == 'recent':
-        # Load recent Kp data
+        unique_fnames = dict()
+        for filename in fnames:
+            fname = filename[0:-11]
+            fdate = dt.datetime.strptime(filename[-10:], '%Y-%m-%d')
+            if fname not in unique_fnames.keys():
+                unique_fnames[fname] = [fdate]
+            else:
+                unique_fnames[fname].append(fdate)
+
+        # Load the desired filenames
+        all_data = []
+        for fname in unique_fnames.keys():
+            # The daily date is attached to the filename.  Parse off the last
+            # date, load the year of data, downselect to the desired day
+            fdate = min(unique_fnames[fname])
+            temp = pds.read_csv(fname, index_col=0, parse_dates=True)
+
+            if temp.empty:
+                pysat.logger.warn('Empty file: {:}'.format(fname))
+                continue
+
+            # Select the desired times and add to data list
+            all_data.append(pds.DataFrame(temp[fdate:max(unique_fnames[fname])
+                                               + dt.timedelta(seconds=86399)]))
+
+        # Combine data together
+        if len(all_data) > 0:
+            result = pds.concat(all_data, axis=0, sort=True)
+        else:
+            result = pds.DataFrame()
+
+        fill_val = np.nan
+    else:
+        # Load the forecast or recent data
         all_data = []
         for fname in fnames:
             result = pds.read_csv(fname, index_col=0, parse_dates=True)
@@ -263,8 +319,50 @@ def load(fnames, tag='', inst_id=''):
         fill_val = -1
 
     # Initalize the meta data
-    for kk in result.keys():
-        kp_ap.initialize_kp_metadata(meta, kk, fill_val)
+    if tag in ['', 'forecast', 'recent']:
+        for kk in result.keys():
+            kp_ap.initialize_kp_metadata(meta, kk, fill_val)
+    else:
+        for kk in result.keys():
+            if kk.find('Kp') >= 0:
+                kp_ap.initialize_kp_metadata(meta, kk, fill_val)
+            elif kk.lower().find('ap') >= 0:
+                kp_ap.initialize_ap_metadata(meta, kk, fill_val)
+
+        meta['Bartels_solar_rotation_num'] = {
+            meta.labels.units: '',
+            meta.labels.name: 'Bartels solar rotation number',
+            meta.labels.desc: ''.join(['A sequence of 27-day intervals counted',
+                                       ' from February 8, 1832']),
+            meta.labels.min_val: 1,
+            meta.labels.max_val: np.inf,
+            meta.labels.fill_val: -1}
+        meta['day_within_Bartels_rotation'] = {
+            meta.labels.units: 'days',
+            meta.labels.name: 'Bartels solar rotation number',
+            meta.labels.desc: ''.join(['Number of day within the Bartels solar',
+                                       ' rotation']),
+            meta.labels.min_val: 1,
+            meta.labels.max_val: 27,
+            meta.labels.fill_val: -1}
+        meta['Cp'] = {
+            meta.labels.units: '',
+            meta.labels.name: 'Cp index',
+            meta.labels.desc: ''.join(['the daily planetary character figure, ',
+                                       'a qualitative estimate of the overall ',
+                                       'level of geomagnetic activity for ',
+                                       'this day determined from the sum of ',
+                                       'the eight ap amplitudes']),
+            meta.labels.min_val: 0.0,
+            meta.labels.max_val: 2.5,
+            meta.labels.fill_val: np.nan}
+        meta['C9'] = {
+            meta.labels.units: '',
+            meta.labels.name: 'C9 index',
+            meta.labels.desc: ''.join(['the contracted scale for Cp']),
+            meta.labels.min_val: 0,
+            meta.labels.max_val: 9,
+            meta.labels.fill_val: -1}
 
     return result, meta
 
@@ -293,28 +391,52 @@ def list_files(tag='', inst_id='', data_path='', format_str=None):
     ----
     Called by pysat. Not intended for direct use by user.
 
+    Warnings
+    --------
+    The '' tag has been deprecated and local file listing support will
+    be removed in version 0.2.0+
+
     """
 
     if tag == '':
+        # This data type has been deprecated due to changes at GFZ
+        warnings.warn("".join(["Changes at the GFZ database have led to this",
+                               " data type being deprecated. Switch to using",
+                               " 'def' for definitive Kp or 'now' for Kp ",
+                               "nowcasts from GFZ. Local file listing support ",
+                               "will be removed in version 0.2.0+"]),
+                      DeprecationWarning, stacklevel=2)
+
         # Files are by month, going to add date to monthly filename for
         # each day of the month. The load routine will load a month of
         # data and use the appended date to select out appropriate data.
         if format_str is None:
             format_str = 'kp{year:2d}{month:02d}.tab'
-        files = pysat.Files.from_os(data_path=data_path,
-                                    format_str=format_str,
-                                    two_digit_year_break=99)
+        files = pysat.Files.from_os(data_path=data_path, format_str=format_str,
+                                    two_digit_year_break=0)
         if not files.empty:
             files.loc[files.index[-1] + pds.DateOffset(months=1)
                       - pds.DateOffset(days=1)] = files.iloc[-1]
             files = files.asfreq('D', 'pad')
             files = files + '_' + files.index.strftime('%Y-%m-%d')
+    elif tag in ['def', 'now']:
+        if format_str is None:
+            format_str = ''.join(['Kp_{:s}'.format(tag), '{year:04d}.txt'])
 
+        # Files are stored by year, going to add a date to the yearly
+        # filename for each month and day of month.  The load routine will load
+        # the year and use the append date to select out approriate data.
+        files = pysat.Files.from_os(data_path=data_path, format_str=format_str)
+        if not files.empty:
+            files.loc[files.index[-1] + pds.DateOffset(years=1)
+                      - pds.DateOffset(days=1)] = files.iloc[-1]
+            files = files.asfreq('D', 'pad')
+            files = files + '_' + files.index.strftime('%Y-%m-%d')
     else:
-        format_str = '_'.join(['kp', tag,
-                               '{year:04d}-{month:02d}-{day:02d}.txt'])
-        files = pysat.Files.from_os(data_path=data_path,
-                                    format_str=format_str)
+        if format_str is None:
+            format_str = '_'.join(['kp', tag,
+                                   '{year:04d}-{month:02d}-{day:02d}.txt'])
+        files = pysat.Files.from_os(data_path=data_path, format_str=format_str)
 
         # Pad list of files data to include most recent file under tomorrow
         if not files.empty:
@@ -339,11 +461,6 @@ def download(date_array, tag, inst_id, data_path):
     data_path : str
         Path to data directory.
 
-    Raises
-    ------
-    Exception
-        Bare raise upon FTP failure, facilitating useful error messages.
-
     Note
     ----
     Called by pysat. Not intended for direct use by user.
@@ -352,63 +469,120 @@ def download(date_array, tag, inst_id, data_path):
     --------
     Only able to download current forecast data, not archived forecasts.
 
+    The '' tag has been deprecated and downloads are no longer supported by
+    the source.  Use 'dep' or 'now' instead.
+
     """
 
     # Download standard Kp data
     if tag == '':
-        ftp = ftplib.FTP('ftp.gfz-potsdam.de')  # connect to host, default port
-        ftp.login()  # user anonymous, passwd anonymous@
-        ftp.cwd('/pub/home/obs/kp-ap/tab')
+        # This data type has been deprecated due to changes at GFZ
+        warnings.warn("".join(["Changes at the GFZ database have led to this",
+                               " data type being deprecated. Switch to using",
+                               " 'def' for definitive Kp or 'now' for Kp ",
+                               "nowcasts from GFZ. Downloads are no longer ",
+                               "supported by GFZ."]),
+                      DeprecationWarning, stacklevel=2)
+    elif tag in ['def', 'now']:
+        # Set the page for the definitive or nowcast Kp
+        burl = ''.join(['https://datapub.gfz-potsdam.de/download/10.5880.Kp.',
+                        '0001/Kp_', 'nowcast' if tag == 'now' else 'definitive',
+                        '/'])
+        data_cols = ['Bartels_solar_rotation_num',
+                     'day_within_Bartels_rotation', 'Kp', 'daily_Kp_sum', 'ap',
+                     'daily_Ap', 'Cp', 'C9']
+        hours = np.arange(0, 24, 3)
+        kp_translate = {'0': 0.0, '3': 1.0 / 3.0, '7': 2.0 / 3.0}
         dnames = list()
 
         for dl_date in date_array:
-            fname = 'kp{year:02d}{month:02d}.tab'
-            fname = fname.format(year=(dl_date.year
-                                       - (dl_date.year // 100) * 100),
-                                 month=dl_date.month)
-            local_fname = fname
-            saved_fname = os.path.join(data_path, local_fname)
+            fname = 'Kp_{:s}{:04d}.wdc'.format(tag, dl_date.year)
             if fname not in dnames:
-                try:
-                    logger.info(' '.join(('Downloading file for',
-                                          dl_date.strftime('%b %Y'))))
-                    sys.stdout.flush()
-                    ftp.retrbinary('RETR ' + fname,
-                                   open(saved_fname, 'wb').write)
+                pysat.logger.info(' '.join(('Downloading file for',
+                                            dl_date.strftime('%Y'))))
+                furl = ''.join([burl, fname])
+                req = requests.get(furl)
+
+                if req.ok:
+                    # Split the file text into lines
+                    lines = req.text.split('\n')[:-1]
+
+                    # Remove the header
+                    while lines[0].find('#') == 0:
+                        lines.pop(0)
+
+                    # Process the data lines
+                    ddict = {dkey: list() for dkey in data_cols}
+                    times = list()
+                    for line in lines:
+                        ldate = dt.datetime.strptime(' '.join([
+                            "{:02d}".format(int(dd)) for dd in
+                            [line[:2], line[2:4], line[4:6]]]), "%y %m %d")
+                        bsr_num = np.int64(line[6:10])
+                        bsr_day = np.int64(line[10:12])
+                        if line[28:30] == '  ':
+                            kp_ones = 0.0
+                        else:
+                            kp_ones = np.float64(line[28:30])
+                        sum_kp = kp_ones + kp_translate[line[30]]
+                        daily_ap = np.int64(line[55:58])
+                        cp = np.float64(line[58:61])
+                        c9 = np.int64(line[61])
+
+                        for i, hour in enumerate(hours):
+                            # Set the time for this hour and day
+                            times.append(ldate + dt.timedelta(hours=int(hour)))
+
+                            # Set the daily values for this hour
+                            ddict['Bartels_solar_rotation_num'].append(bsr_num)
+                            ddict['day_within_Bartels_rotation'].append(bsr_day)
+                            ddict['daily_Kp_sum'].append(sum_kp)
+                            ddict['daily_Ap'].append(daily_ap)
+                            ddict['Cp'].append(cp)
+                            ddict['C9'].append(c9)
+
+                            # Get the hourly-specific values
+                            ikp = i * 2
+                            kp_ones = line[12 + ikp]
+                            if kp_ones == ' ':
+                                kp_ones = 0.0
+                            ddict['Kp'].append(np.float64(kp_ones)
+                                               + kp_translate[line[13 + ikp]])
+                            iap = i * 3
+                            ddict['ap'].append(np.int64(
+                                line[31 + iap:34 + iap]))
+
+                    # Put data into nicer DataFrame
+                    data = pds.DataFrame(ddict, index=times, columns=data_cols)
+
+                    # Write out as a CSV file
+                    saved_fname = os.path.join(data_path, fname).replace(
+                        '.wdc', '.txt')
+                    data.to_csv(saved_fname, header=True)
+
+                    # Record the filename so we don't download it twice
                     dnames.append(fname)
-                except ftplib.error_perm as exception:
-
-                    if str(exception.args[0]).split(" ", 1)[0] != '550':
-                        # Leaving a bare raise below so that ftp errors
-                        # are properly reported as coming from ftp
-                        # and gives the correct line number.
-                        # We aren't expecting any 'normal' ftp errors
-                        # here, other than a 550 'no file' error, thus
-                        # accurately raising FTP issues is the way to go
-                        raise
-                    else:
-                        # File isn't actually there, just let people know
-                        # then continue on
-                        os.remove(saved_fname)
-                        logger.info(' '.join(('File not available for',
-                                              dl_date.strftime('%x'))))
-
-        ftp.close()
+                else:
+                    pysat.logger.info("".join(["Unable to download data for ",
+                                               dl_date.strftime("%d %b %Y"),
+                                               ", date may be out of range ",
+                                               "for the database."]))
 
     elif tag == 'forecast':
-        logger.info(' '.join(('This routine can only download the current',
-                              'forecast, not archived forecasts')))
+        pysat.logger.info(' '.join(('This routine can only download the ',
+                                    'current forecast, not archived ',
+                                    'forecasts')))
 
         # Download webpage
         furl = 'https://services.swpc.noaa.gov/text/3-day-geomag-forecast.txt'
-        r = requests.get(furl)
+        req = requests.get(furl)
 
         # Parse text to get the date the prediction was generated
-        date_str = r.text.split(':Issued: ')[-1].split(' UTC')[0]
+        date_str = req.text.split(':Issued: ')[-1].split(' UTC')[0]
         dl_date = dt.datetime.strptime(date_str, '%Y %b %d %H%M')
 
         # Data is the forecast value for the next three days
-        raw_data = r.text.split('NOAA Kp index forecast ')[-1]
+        raw_data = req.text.split('NOAA Kp index forecast ')[-1]
 
         # Get date of the forecasts
         date_str = raw_data[0:6] + ' ' + str(dl_date.year)
@@ -425,9 +599,10 @@ def download(date_array, tag, inst_id, data_path):
         day3 = []
         for line in lines:
             raw = raw_data.split(line)[-1].split('\n')[0]
-            day1.append(int(raw[0:10]))
-            day2.append(int(raw[10:20]))
-            day3.append(int(raw[20:]))
+            cols = raw.split()
+            day1.append(np.float64(cols[-3]))
+            day2.append(np.float64(cols[-2]))
+            day3.append(np.float64(cols[-1]))
 
         times = pds.date_range(forecast_date, periods=24, freq='3H')
         day = []
@@ -442,8 +617,8 @@ def download(date_array, tag, inst_id, data_path):
         data.to_csv(os.path.join(data_path, data_file), header=True)
 
     elif tag == 'recent':
-        logger.info(' '.join(('This routine can only download the current',
-                              'webpage, not archived forecasts')))
+        pysat.logger.info(' '.join(('This routine can only download the ',
+                                    'current webpage, not archived forecasts')))
 
         # Download webpage
         rurl = ''.join(('https://services.swpc.noaa.gov/text/',
@@ -470,11 +645,19 @@ def download(date_array, tag, inst_id, data_path):
         for line in raw_data:
             kp_time.append(dt.datetime.strptime(line[0:10], '%Y %m %d'))
 
-            # Pick out Kp values for each of the three columns
+            # Pick out Kp values for each of the three columns. The columns
+            # used to all have integer values, but now some have floats.
             sub_lines = [line[17:33], line[40:56], line[63:]]
-            for sub_line, sub_kp in zip(sub_lines, sub_kps):
-                for i in np.arange(8):
-                    sub_kp.append(int(sub_line[(i * 2):((i + 1) * 2)]))
+            for i, sub_line in enumerate(sub_lines):
+                split_sub = sub_line.split()
+                for ihr in np.arange(8):
+                    if sub_line.find('.') < 0:
+                        # These are integer values
+                        sub_kps[i].append(
+                            np.int64(sub_line[(ihr * 2):((ihr + 1) * 2)]))
+                    else:
+                        # These are float values
+                        sub_kps[i].append(np.float64(split_sub[ihr]))
 
         # Create times on 3 hour cadence
         times = pds.date_range(kp_time[0], periods=(8 * 30), freq='3H')
